module debounce (
    input      clk_i,
    input      rst_i,
    input      in_i,  // bouncing button input
    output reg out_o  // 1 cycle pulse for every button press
);

    // implement the debounce circuit here
    reg [1:0] cstate, nstate;
    reg [16:0] counter, count;

    always @(posedge clk_i or posedge rst_i) begin
        if (rst_i) begin
            cstate <= 2'b0;
            counter <= 17'b0;
        end else begin
            cstate <= nstate;
            counter <= count;
        end
    end

    always @(*) begin
        out_o = 1'b0;
        case(cstate)
        2'b00: begin
            count = 17'b0;
            if (in_i) begin
                nstate = 2'b01;
            end else begin
                nstate = 2'b00;
            end
        end
        2'b01: begin
            if (counter == 17'd125000) begin
                out_o = 1'b1;
                nstate = 2'b10;
                count = 17'b0;
            end else if (in_i) begin
                nstate = 2'b01;
                count = counter + 17'b1;
            end else begin
                nstate = 2'b00;
                count = 17'b0;
            end
        end
        2'b10: begin
            count = 17'b0;
            if (!in_i) begin
                nstate = 2'b11;
            end else begin
                nstate = 2'b10;
            end
        end
        2'b11: begin
            if (counter == 17'd125000) begin
                nstate = 2'b00;
                count = 17'b0;
            end else if (!in_i) begin
                nstate = 2'b11;
                count = counter + 17'b1;
            end else begin
                nstate = 2'b10;
                count = 17'b0;
            end
        end
        default: begin
            nstate = 2'b00;
            count = 17'b0;
        end
        endcase
    end

endmodule