module stack(
    input       clk,
    input       rst,
    input [1:0] operation,
    input [7:0] in,
    // Register
    output reg [7:0] out,
    output       empty,
    // Wire
    output       full
);

    reg [7:0] mem[0:15];
    reg [4:0] sp, sp_d;

    assign empty = (sp == 5'b11111);
    assign full = (sp == 5'b01111);

    always @(posedge clk or posedge rst) begin
        if (rst) begin
            sp <= 5'b11111;
            out <= 8'b0;
        end else begin
            sp <= sp_d;            
            if (operation == 2'b10) begin
                out <= mem[sp];
            end else if (operation == 2'b01) begin
                mem[sp+1] <= in;
            end
        end
    end

    always @(*) begin : next_sp
        case(operation)
        2'b00: begin
            sp_d = sp;             
        end
        2'b01: begin
            if (!full) sp_d = sp + 5'b1;
            else sp_d = sp;
        end
        2'b10: begin
            if (!empty) sp_d = sp - 5'b1;
            else sp_d = sp;
        end
        2'b11: begin
            sp_d = 5'b11111;
        end
        default: begin
            sp_d = 5'b0;
        end
        endcase
    end
endmodule