module top (
    input sysclk,
    input [3:0] btn,
    output [13:0] ar,
    output [2:0] led
);

    wire in0, in1, in2;
    wire [7:0] stack_in, stack_out, result;
    wire [1:0] stack_op;
    wire [6:0] seg70_out, seg71_out;
    
    assign ar[6:0] = seg70_out;
    assign ar[13:7] = seg71_out;
    
    debounce debounce0 (
        .clk_i(sysclk),
        .rst_i(btn[3]),
        .in_i(btn[0]),
        .out_o(in0) 
    );

    debounce debounce1 (
        .clk_i(sysclk),
        .rst_i(btn[3]),
        .in_i(btn[1]),
        .out_o(in1) 
    );

    debounce debounce2 (
        .clk_i(sysclk),
        .rst_i(btn[3]),
        .in_i(btn[2]),
        .out_o(in2) 
    );

    stack stack0 (
        .clk(sysclk),
        .rst(btn[3]),
        .operation(stack_op),
        .in(stack_in),
        .out(stack_out),
        .empty(),
        .full()
    );

    calculator calculator0 (
        .clk_i(sysclk),
        .rst_i(btn[3]),
        .in_i({in2, in1, in0}),
        .stack_out_i(stack_out),
        .stack_op_o(stack_op),
        .stack_in_o(stack_in),
        .result_o(result),
        .led_o(led)
    );

    seg7 seg70 (
        .num_i(result[3:0]),
        .seg7_o(seg70_out)
    );

    seg7 seg71 (
        .num_i(result[7:4]),
        .seg7_o(seg71_out)
    );

endmodule